/* This file is part of Alienwave, a game by Alessandro Pira */

#ifndef _LEVELS_H
#define _LEVELS_H

#define MAX_LEVEL_SIZE 100

void init_levels();

// Returns 0 if the game is over (Xzarna died)
int create_aliens();

#endif //_LEVELS_H
